@@ -1,6 +1,7 @@
 from .computation_engine import (apply, join, summarize,
                                  apply_local, join_local, summarize_local,
-                                 Engine, output, DelayedException)
+                                 Engine, output, DelayedException,
+                                 fullstream, fullstream_local)
 
 from .fs_cache import CacheBuilder, LocalCacheBuilder
 
