@@ -330,6 +330,40 @@ class SummarizeComputation(Computation):
         self._start_async_computation_for(0, (arg,))
 
 
+class FullStreamComputation(Computation):
+
+    def __init__(self, f, inputs, run_local=False):
+        assert len(inputs) == 1, "full stream computations only accept one input stream"
+        super(FullStreamComputation, self).__init__(f, inputs, run_local)
+
+    def compute_size(self):
+        return self.input_streams[0].size()
+
+    def _start_computation(self):
+        items = []
+        stream = self.input_streams[0]
+        for i in range(stream.size()):
+            item = stream.get(listener=self)
+            if isinstance(item.value, DelayedException):
+                self.put(item)
+                return
+            items.append(item)
+        items.sort(key=lambda item: item.number)
+        values = [v for __, v in items]
+        # arg = EnumeratedItem(0, values)
+        # self._start_async_computation_for(0, (arg,))
+        for i, value in enumerate(values):
+            if isinstance(value, DelayedException):
+                wrap_callback(i, self.put)(value)
+                return
+
+        def forward_result(result):
+            for i, item in enumerate(result):
+                self.put(EnumeratedItem(i, item))
+
+        Engine.run_async(self.f, (values,), forward_result, self.run_local)
+
+
 class ZipComputation(Computation):
 
     """either input_streams provide all n inputs or one fixed input. The latter is managed by
@@ -434,6 +468,7 @@ def apply_local(inner):
     wrapped.inner = inner
     return wrapped
 
+
 def join(inner):
     @functools.wraps(inner)
     def wrapped(*args):
@@ -441,6 +476,7 @@ def join(inner):
     wrapped.inner = inner
     return wrapped
 
+
 def join_local(inner):
     @functools.wraps(inner)
     def wrapped(*args):
@@ -448,6 +484,7 @@ def join_local(inner):
     wrapped.inner = inner
     return wrapped
 
+
 def summarize(inner):
     @functools.wraps(inner)
     def wrapped(*args):
@@ -455,6 +492,7 @@ def summarize(inner):
     wrapped.inner = inner
     return wrapped
 
+
 def summarize_local(inner):
     @functools.wraps(inner)
     def wrapped(*args):
@@ -462,6 +500,23 @@ def summarize_local(inner):
     wrapped.inner = inner
     return wrapped
 
+
+def fullstream(inner):
+    @functools.wraps(inner)
+    def wrapped(*args):
+        return FullStreamComputation(inner, args)
+    wrapped.inner = inner
+    return wrapped
+
+
+def fullstream_local(inner):
+    @functools.wraps(inner)
+    def wrapped(*args):
+        return FullStreamComputation(inner, args, run_local=True)
+    wrapped.inner = inner
+    return wrapped
+
+
 def output(inner):
     if isinstance(inner, Computation):
         return OutputNode(inner)
